/*
    zr36120_mem.c - Generic memory support for video devices

    No Copyright by Pauline Middelink <middelin@polyware.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <linux/mm.h>
#include <linux/version.h>
#include <linux/pci.h>
#include <linux/wrapper.h>
#include <linux/slab.h>
#include <asm/io.h>
#ifdef CONFIG_BIGPHYS_AREA
#include <linux/bigphysarea.h>
#endif

#include "zr36120.h"
#include "zr36120_mem.h"

/*******************************/
/* Memory management functions */
/*******************************/
void* bmalloc(unsigned long size)
{
	void* mem;
	size = PAGE_ALIGN(size);
#ifdef CONFIG_BIGPHYS_AREA
	mem = bigphysarea_alloc_pages(size>>PAGE_SHIFT, 1, GFP_KERNEL);
#else
//#error	zoran driver requires bigphysarea patch (see README)
	mem = kmalloc(size, GFP_KERNEL);
#endif
	if (mem) {
		ulong adr = (ulong)mem;
memset(mem,0x4A,size);
		while (size) {
			mem_map_reserve(MAP_NR(adr));
			adr += PAGE_SIZE;
			size -= PAGE_SIZE;
		}
	}
	return mem;
}

void bfree(void* mem, unsigned long size)
{
	if (mem) {
		ulong adr = (ulong)mem;
		size = PAGE_ALIGN(size);
		while (size) {
			mem_map_unreserve(MAP_NR(adr));
			adr += PAGE_SIZE;
			size -= PAGE_SIZE;
		}
#ifdef CONFIG_BIGPHYS_AREA
		bigphysarea_free_pages(mem);
#else
		kfree(mem);
#endif
	}
}
